@@ -1,8 +1,7 @@
 /***************************************************************************
  * Basic partition manager for the installer.
- ***************************************************************************
  *
- *   Copyright (C) 2019, 2020-2022 by AK-47
+ *   Copyright (C) 2019-2023 by AK-47
  *   Transplanted code, marked with comments further down this file:
  *    - Copyright (C) 2003-2010 by Warren Woodford
  *    - Heavily edited, with permision, by anticapitalista for antiX 2011-2014.
@@ -22,11 +21,11 @@
  * This file is part of the gazelle-installer.
  ***************************************************************************/
 
-#include <sys/sysinfo.h>
 #include <sys/stat.h>
 #include <QDebug>
 #include <QLocale>
 #include <QFileInfo>
+#include <QDir>
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QJsonArray>
@@ -40,26 +39,24 @@
 #include <QFormLayout>
 #include <QDialogButtonBox>
 
+#include "mprocess.h"
 #include "msettings.h"
+#include "autopart.h"
 #include "partman.h"
 
-#define PARTMAN_SAFETY_MB 8 // 1MB at start + Compensate for rounding errors.
-#define PARTMAN_MAX_PARTS 128 // Maximum number of partitions Linux supports.
-
 PartMan::PartMan(MProcess &mproc, Ui::MeInstall &ui, const QSettings &appConf, const QCommandLineParser &appArgs)
     : QAbstractItemModel(ui.boxMain), proc(mproc), root(DeviceItem::Unknown), gui(ui)
 {
     const QString &projShort = appConf.value("PROJECT_SHORTNAME").toString();
-    defaultLabels["ESP"] = "EFI System";
-    defaultLabels["BIOS-GRUB"] = "BIOS GRUB";
-    defaultLabels["/boot"] = "boot";
-    defaultLabels["/"] = "root" + projShort + appConf.value("VERSION").toString();
-    defaultLabels["/home"] = "home" + projShort;
-    defaultLabels["SWAP"] = "swap" + projShort;
+    volSpecs["ESP"] = {"EFI System"};
+    volSpecs["BIOS-GRUB"] = {"BIOS GRUB"};
+    volSpecs["/boot"] = {"boot"};
+    volSpecs["/"] = {"root" + projShort + appConf.value("VERSION").toString()};
+    volSpecs["/home"] = {"home" + projShort};
+    volSpecs["SWAP"] = {"swap" + projShort};
     brave = appArgs.isSet("brave");
     gptoverride = appArgs.isSet("gpt-override");
 
-    master = gui.boxMain;
     root.partman = this;
     gui.treePartitions->setModel(this);
     gui.treePartitions->setItemDelegate(new DeviceItemDelegate);
@@ -152,6 +149,7 @@ void PartMan::scan(DeviceItem *drvstart)
             partit->flags.oldLayout = true;
             partit->device = jsonPart["name"].toString();
             partit->path = jsonPart["path"].toString();
+            partit->uuid = jsonPart["uuid"].toString();
             partit->order = order.indexOf(partit->device);
             partit->size = jsonPart["size"].toVariant().toLongLong();
             partit->physec = jsonPart["phy-sec"].toInt();
@@ -163,7 +161,7 @@ void PartMan::scan(DeviceItem *drvstart)
             if ((partflags & 0x80) || (partflags & 0x04)) partit->setActive(true);
             partit->mapCount = jsonPart["children"].toArray().count();
             partit->flags.curESP = partTypeName.startsWith("EFI "); // "System"/"(FAT-12/16/32)"
-            partit->flags.bootRoot = (!bootUUID.isEmpty() && jsonPart["uuid"] == bootUUID);
+            partit->flags.bootRoot = (!bootUUID.isEmpty() && partit->uuid == bootUUID);
             partit->curFormat = jsonPart["fstype"].toString();
             if (partTypeName == "BIOS boot") partit->curFormat = "BIOS-GRUB";
             // Touching MacOS first drive, or MS LDM may brick the system.
@@ -254,11 +252,12 @@ void PartMan::scanVirtualDevices(bool rescan)
             devit = new DeviceItem(DeviceItem::VirtualBD, vdlit);
             devit->device = jsonDev["name"].toString();
             devit->path = path;
+            devit->uuid = jsonDev["uuid"].toString();
             devit->flags.rotational = rota;
             devit->discgran = discgran;
             devit->size = size;
             devit->physec = physec;
-            devit->flags.bootRoot = (!bootUUID.isEmpty() && jsonDev["uuid"] == bootUUID);
+            devit->flags.bootRoot = (!bootUUID.isEmpty() && devit->uuid == bootUUID);
             devit->curLabel = label;
             devit->curFormat = jsonDev["fstype"].toString();
             devit->flags.volCrypto = crypto;
@@ -305,7 +304,7 @@ bool PartMan::manageConfig(MSettings &config, bool save)
             if (partCount > PARTMAN_MAX_PARTS) return false;
         }
         // Partition configuration.
-        const long long sizeMax = drvit->size - (PARTMAN_SAFETY_MB * 1048576);
+        const long long sizeMax = drvit->size - PARTMAN_SAFETY;
         long long sizeTotal = 0;
         for (int ixPart = 0; ixPart < partCount; ++ixPart) {
             DeviceItem *partit = nullptr;
@@ -429,7 +428,7 @@ void PartMan::treeSelChange()
         if (!islocked && isold && isdrive) gui.pushPartAdd->setEnabled(false);
         else if (!isold) {
             gui.pushPartAdd->setEnabled(drvit->childCount() < PARTMAN_MAX_PARTS
-                && twit->driveFreeSpace(true) > 1048576);
+                && twit->driveFreeSpace(true) > 1*MB);
         }
     } else {
         gui.pushPartClear->setEnabled(false);
@@ -507,22 +506,23 @@ void PartMan::treeMenu(const QPoint &)
         menu.addSeparator();
         QAction *actClear = menu.addAction(tr("New &layout"));
         QAction *actReset = menu.addAction(tr("&Reset layout"));
-        QMenu *menuTemplates = menu.addMenu(tr("&Templates"));
-        const QAction *actBasic = menuTemplates->addAction(tr("&Standard install"));
-        QAction *actCrypto = menuTemplates->addAction(tr("&Encrypted system"));
+        menu.addSeparator();
+        QAction *actBuilder = menu.addAction(tr("Layout &Builder..."));
 
         const bool locked = twit->isLocked();
         actClear->setDisabled(locked);
         actReset->setDisabled(locked);
-        menuTemplates->setDisabled(locked);
-        actCrypto->setVisible(gui.boxCryptoPass->isVisible());
+
+        const long long minSpace = brave ? 0 : volSpecTotal("/", QStringList()).minimum;
+        actBuilder->setEnabled(!locked && autopart && twit->size >= minSpace);
 
         QAction *action = menu.exec(QCursor::pos());
         if (action == actAdd) partAddClick(true);
         else if (action == actClear) partClearClick(true);
-        else if (action == actBasic) twit->layoutDefault(-1, false);
-        else if (action == actCrypto) twit->layoutDefault(-1, true);
-        else if (action == actReset) {
+        else if (action == actBuilder) {
+            if (autopart) autopart->builderGUI(twit);
+            treeSelChange();
+        } else if (action == actReset) {
             gui.boxMain->setEnabled(false);
             scan(twit);
             gui.boxMain->setEnabled(true);
@@ -595,7 +595,7 @@ void PartMan::partManRunClick()
 // Partition menu items
 void PartMan::partMenuUnlock(DeviceItem *twit)
 {
-    QDialog dialog(master);
+    QDialog dialog(gui.boxMain);
     QFormLayout layout(&dialog);
     dialog.setWindowTitle(tr("Unlock Drive"));
     QLineEdit *editVDev = new QLineEdit(&dialog);
@@ -632,7 +632,7 @@ void PartMan::partMenuUnlock(DeviceItem *twit)
             qDebug() << "partMenuUnlock() failed:" << msg;
             gui.boxMain->setEnabled(true);
             qApp->restoreOverrideCursor();
-            QMessageBox::warning(master, QString(),
+            QMessageBox::warning(gui.boxMain, QString(),
                 tr("Could not unlock device. Possible incorrect password."));
         }
     }
@@ -662,7 +662,7 @@ void PartMan::partMenuLock(DeviceItem *twit)
     qApp->restoreOverrideCursor();
     // If not OK then a command failed, or trying to close a non-LUKS device.
     if (!ok) {
-        QMessageBox::critical(master, QString(), tr("Failed to close %1").arg(twit->device));
+        QMessageBox::critical(gui.boxMain, QString(), tr("Failed to close %1").arg(twit->device));
     }
 }
 
@@ -706,7 +706,7 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
             if (cmptext.count(QRegularExpression("[^A-Z0-9\\/\\@\\.\\-\\_]|\\/\\/"))) ok = false;
             if (cmptext.startsWith('/') || cmptext.endsWith('/')) ok = false;
             if (!ok) {
-                QMessageBox::critical(master, QString(), tr("Invalid subvolume label"));
+                QMessageBox::critical(gui.boxMain, QString(), tr("Invalid subvolume label"));
                 return false;
             }
             // Check for duplicate subvolume label entries.
@@ -717,7 +717,7 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
             for (int ixi = 0; ixi < count; ++ixi) {
                 if (ixi == index) continue;
                 if (pit->child(ixi)->label.trimmed().toUpper() == cmptext) {
-                    QMessageBox::critical(master, QString(), tr("Duplicate subvolume label"));
+                    QMessageBox::critical(gui.boxMain, QString(), tr("Duplicate subvolume label"));
                     return false;
                 }
             }
@@ -725,7 +725,7 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
         QString mount = item->realUseFor();
         if (mount.isEmpty()) continue;
         if (!mount.startsWith("/") && !item->allowedUsesFor().contains(mount)) {
-            QMessageBox::critical(master, QString(),
+            QMessageBox::critical(gui.boxMain, QString(),
                 tr("Invalid use for %1: %2").arg(item->shownDevice(), mount));
             return false;
         }
@@ -737,7 +737,7 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
 
         // The mount can only be selected once.
         if (twit) {
-            QMessageBox::critical(master, QString(), tr("%1 is already"
+            QMessageBox::critical(gui.boxMain, QString(), tr("%1 is already"
                 " selected for: %2").arg(twit->shownDevice(), twit->shownUseFor()));
             return false;
         } else if(!mount.isEmpty() && mount != "FORMAT" && mount != "ESP" && mount != "BIOS-GRUB"){
@@ -758,23 +758,25 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
     }
 
     DeviceItem *rootitem = mounts.value("/");
-    if (!rootitem || rootitem->size < rootSpaceNeeded) {
-        const QString &tmin = QLocale::system().formattedDataSize(rootSpaceNeeded + 1048575,
-            1, QLocale::DataSizeTraditionalFormat);
-        QMessageBox::critical(master, QString(),
-            tr("A root partition of at least %1 is required.").arg(tmin));
+    DeviceItem *bootitem = mounts.value("/boot");
+    const long long rootMin = volSpecTotal("/").minimum;
+    const QString &tMinRoot = QLocale::system().formattedDataSize(rootMin,
+        1, QLocale::DataSizeTraditionalFormat);
+    if (!rootitem) {
+        QMessageBox::critical(gui.boxMain, QString(),
+            tr("A root partition of at least %1 is required.").arg(tMinRoot));
         return false;
     } else {
         if (!rootitem->willFormat() && mounts.contains("/home")) {
             const QString errmsg = tr("Cannot preserve /home inside root (/) if"
                 " a separate /home partition is also mounted.");
-            QMessageBox::critical(master, QString(), errmsg);
+            QMessageBox::critical(gui.boxMain, QString(), errmsg);
             return false;
         }
         if (rootitem->encrypt) encryptRoot = true;
     }
-    if (encryptRoot && !mounts.contains("/boot")) {
-        QMessageBox::critical(master, QString(),
+    if (encryptRoot && !bootitem) {
+        QMessageBox::critical(gui.boxMain, QString(),
             tr("You must choose a separate boot partition when encrypting root."));
         return false;
     }
@@ -782,6 +784,8 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
     if (!automatic) {
         // Final warnings before the installer starts.
         QString details, biosgpt;
+        QStringList tooSmall;
+        const QString &msgszItem(" - " + tr("%1 (%2) requires %3"));
         for (int ixdrv = 0; ixdrv < root.childCount(); ++ixdrv) {
             DeviceItem *drvit = root.child(ixdrv);
             const int partCount = drvit->childCount();
@@ -808,6 +812,14 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
                 }
                 details += actmsg.arg(partit->shownDevice(), partit->shownUseFor()) + '\n';
                 if (use == "BIOS-GRUB") hasBiosGrub = true;
+
+                // If this volume is too small, add to the warning list.
+                const long long minSize = volSpecTotal(partit->realUseFor()).minimum;
+                if (partit->size < minSize) {
+                    const QString &mstr = QLocale::system().formattedDataSize(minSize,
+                        1, QLocale::DataSizeTraditionalFormat);
+                    tooSmall.append(msgszItem.arg(partit->shownUseFor(), partit->device, mstr));
+                }
             }
             // Potentially unbootable GPT when on a BIOS-based PC.
             const bool hasBoot = (drvit->active != nullptr);
@@ -816,10 +828,18 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
             }
         }
         // Warning messages
-        QMessageBox msgbox(master);
+        QMessageBox msgbox(gui.boxMain);
         msgbox.setIcon(QMessageBox::Warning);
         msgbox.setStandardButtons(QMessageBox::Yes|QMessageBox::No);
         msgbox.setDefaultButton(QMessageBox::No);
+        if (!tooSmall.isEmpty()) {
+            tooSmall.sort();
+            msgbox.setText(tr("The installation may fail because"
+                " the following volumes are too small:") + "\n\n"
+                + tooSmall.join('\n')
+                + "\n\n" + tr("Are you sure you want to continue?"));
+            if (msgbox.exec() != QMessageBox::Yes) return false;
+        }
         if (!biosgpt.isEmpty()) {
             biosgpt.prepend(tr("The following drives are, or will be, setup with GPT,"
                 " but do not have a BIOS-GRUB partition:") + "\n\n");
@@ -830,7 +850,15 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
         }
         msgbox.setText(tr("The %1 installer will now perform the requested actions.").arg(project));
         msgbox.setInformativeText(tr("These actions cannot be undone. Do you want to continue?"));
+        details.chop(1); // Remove trailing new-line character.
         msgbox.setDetailedText(details);
+        const QList<QAbstractButton *> &btns = msgbox.buttons();
+        for (auto btn : btns) {
+            if (msgbox.buttonRole(btn) == QMessageBox::ActionRole) {
+                btn->click(); // Simulate clicking the "Show Details..." button.
+                break;
+            }
+        }
         if (msgbox.exec() != QMessageBox::Yes) return false;
     }
 
@@ -881,12 +909,12 @@ bool PartMan::checkTargetDrivesOK()
         msg += tr("If unsure, please exit the Installer and run GSmartControl for more information.") + "\n\n";
         if (!smartFail.isEmpty()) {
             msg += tr("Do you want to abort the installation?");
-            ans = QMessageBox::critical(master, QString(), msg,
+            ans = QMessageBox::critical(gui.boxMain, QString(), msg,
                     QMessageBox::Yes|QMessageBox::Default|QMessageBox::Escape, QMessageBox::No);
             if (ans == QMessageBox::Yes) return false;
         } else {
             msg += tr("Do you want to continue?");
-            ans = QMessageBox::warning(master, QString(), msg,
+            ans = QMessageBox::warning(gui.boxMain, QString(), msg,
                     QMessageBox::Yes|QMessageBox::Default, QMessageBox::No|QMessageBox::Escape);
             if (ans != QMessageBox::Yes) return false;
         }
@@ -896,7 +924,7 @@ bool PartMan::checkTargetDrivesOK()
 
 void PartMan::luksFormat(DeviceItem *partit, const QByteArray &password)
 {
-    QStringList cargs({"--batch-mode", "--key-size=512", "--hash=sha512", "--pbkdf=argon2id"});
+    QStringList cargs({"--batch-mode", "--key-size=512", "--hash=sha512"});
     if (partit->physec > 0) cargs.append("--sector-size=" + QString::number(partit->physec));
     cargs << "luksFormat" << partit->path;
     if (!proc.exec("cryptsetup", cargs, &password)) {
@@ -955,6 +983,33 @@ int PartMan::countPrepSteps()
     }
     return nstep;
 }
+void PartMan::prepStorage()
+{
+    preparePartitions();
+    formatPartitions();
+    mountPartitions();
+    // Run blkid -c /dev/null to freshen UUID cache
+    proc.exec("blkid", {"-c", "/dev/null"});
+    // Refresh the UUIDs
+    proc.exec("lsblk", {"--list", "-bJo", "PATH,UUID"}, nullptr, true);
+    const QJsonObject &jsonObjBD = QJsonDocument::fromJson(proc.readOut(true).toUtf8()).object();
+    const QJsonArray &jsonBD = jsonObjBD["blockdevices"].toArray();
+    QMap<QString, DeviceItem *> alldevs;
+    for (DeviceItemIterator it(*this); DeviceItem *item = *it; it.next()) {
+        alldevs.insert(item->path, item);
+    }
+    for (const QJsonValue &jsonDev : jsonBD) {
+        DeviceItem *devit = alldevs.value(jsonDev["path"].toString());
+        if (devit) devit->uuid = jsonDev["uuid"].toString();
+    }
+}
+void PartMan::installTabs()
+{
+    makeFstab();
+    if (!fixCryptoSetup()) {
+        throw QT_TR_NOOP("Failed to finalize encryption setup.");
+    }
+}
 
 void PartMan::preparePartitions()
 {
@@ -1033,7 +1088,7 @@ void PartMan::preparePartitions()
             long long start = 1; // start with 1 MB to aid alignment
             for (int ixdev = 0; ixdev<devCount; ++ixdev) {
                 DeviceItem *twit = drvit->child(ixdev);
-                const long long end = start + twit->size / 1048576;
+                const long long end = start + twit->size / MB;
                 const bool rc = proc.exec("parted", {"-s", "--align", "optimal", drvit->path,
                     "mkpart" , "primary", QString::number(start) + "MiB", QString::number(end) + "MiB"});
                 if (!rc) throw msgfail;
@@ -1235,8 +1290,7 @@ bool PartMan::fixCryptoSetup()
     QTextStream out(&file);
     // Add devices to crypttab.
     for (auto &it : cryptAdd.toStdMap()) {
-        proc.exec("blkid", {"-s", "UUID", "-o", "value", "/dev/" + it.first}, nullptr, true);
-        out << it.second->devMapper << " /dev/disk/by-uuid/" << proc.readOut();
+        out << it.second->devMapper << " /dev/disk/by-uuid/" << it.second->uuid;
         if (noKey || it.first == keyDev) out << " none";
         else {
             if (isNewKey) {
@@ -1262,7 +1316,7 @@ bool PartMan::fixCryptoSetup()
 }
 
 // create /etc/fstab file
-bool PartMan::makeFstab(bool populateMediaMounts)
+bool PartMan::makeFstab()
 {
     QFile file("/mnt/antiX/etc/fstab");
     if (!file.open(QIODevice::WriteOnly)) return false;
@@ -1275,10 +1329,7 @@ bool PartMan::makeFstab(bool populateMediaMounts)
         qDebug() << "Creating fstab entry for:" << it.first << dev;
         // Device ID or UUID
         if (twit->willMap()) out << dev;
-        else {
-            proc.exec("blkid", {"-o", "value", "UUID", "-s", "UUID", dev}, nullptr, true);
-            out << "UUID=" + proc.readOut();
-        }
+        else out << "UUID=" << twit->assocUUID();
         // Mount point, file system
         proc.exec("blkid", {dev, "-o", "value", "-s", "TYPE"}, nullptr, true);
         const QString &fsfmt = proc.readOut();
@@ -1303,15 +1354,11 @@ bool PartMan::makeFstab(bool populateMediaMounts)
     }
     // EFI System Partition
     if (gui.radioBootESP->isChecked()) {
-        const QString &dev = "/dev/" + gui.comboBoot->currentData().toString();
-        proc.exec("blkid", {"-o", "value", "UUID", "-s", "UUID", dev}, nullptr, true);
-        out << "UUID=" << proc.readOut() << " /boot/efi vfat noatime,dmask=0002,fmask=0113 0 0\n";
+        DeviceItem *devit = findByPath("/dev/" + gui.comboBoot->currentData().toString());
+        if (!devit) return false;
+        out << "UUID=" << devit->uuid << " /boot/efi vfat noatime,dmask=0002,fmask=0113 0 0\n";
     }
     file.close();
-    if (populateMediaMounts) {
-        if (!proc.shell("/sbin/make-fstab -O --install=/mnt/antiX"
-            " --mntpnt=/media")) return false;
-    }
     return true;
 }
 
@@ -1415,6 +1462,30 @@ DeviceItem *PartMan::findByPath(const QString &devpath) const
     }
     return nullptr;
 }
+DeviceItem *PartMan::findHostDev(const QString &path) const
+{
+    QString spath = path;
+    DeviceItem *devit = nullptr;
+    do {
+        spath = QDir(QFileInfo(spath).absolutePath()).path();
+        devit = mounts.value(spath);
+    } while(!devit && !spath.isEmpty() && spath != '/');
+    return devit;
+}
+
+struct PartMan::VolumeSpec PartMan::volSpecTotal(const QString &path, const QStringList &vols) const
+{
+    struct VolumeSpec vspec = volSpecs.value(path);
+    const QString &spath = (path!='/' ? path+'/' : path);
+    for (const auto &it : volSpecs.toStdMap()) {
+        if (it.first.size() > 1 && it.first.startsWith(spath) && !vols.contains(it.first)) {
+            vspec.image += it.second.image;
+            vspec.minimum += it.second.minimum;
+            vspec.preferred += it.second.preferred;
+        }
+    }
+    return vspec;
+}
 
 /*************************\
  * Model View Controller *
@@ -1698,7 +1769,7 @@ void PartMan::notifyChange(class DeviceItem *item, int first, int last)
 DeviceItem::DeviceItem(enum DeviceType type, DeviceItem *parent, DeviceItem *preceding)
     : parentItem(parent), type(type)
 {
-    if (type == Partition) size = 1048576;
+    if (type == Partition) size = 1*MB;
     if (parent) {
         flags.rotational = parent->flags.rotational;
         discgran = parent->discgran;
@@ -1788,9 +1859,8 @@ QString DeviceItem::realUseFor(const QString &use)
     else if (!use.startsWith('/')) return use.toUpper();
     else return use;
 }
-QString DeviceItem::shownUseFor() const
+QString DeviceItem::shownUseFor(const QString &use)
 {
-    const QString &use = realUseFor();
     if (use == "/") return "/ (root)";
     else if (use == "ESP") return tr("EFI System Partition");
     else if (use == "SWAP") return tr("swap space");
@@ -1819,11 +1889,11 @@ bool DeviceItem::isLocked() const
     }
     return (mapCount != 0);
 }
-inline bool DeviceItem::willUseGPT() const
+bool DeviceItem::willUseGPT() const
 {
     if (type != Drive) return false;
     if (flags.oldLayout) return flags.curGPT;
-    else if (size >= 2199023255552 || children.count() > 4) return true;
+    else if (size >= 2*TB || children.count() > 4) return true;
     else if (partman) return (partman->gptoverride || partman->proc.detectEFI());
     return false;
 }
@@ -1842,6 +1912,11 @@ inline bool DeviceItem::willEncrypt() const
     if (type == Subvolume) return parentItem->encrypt;
     return encrypt;
 }
+QString DeviceItem::assocUUID() const
+{
+    if (type == Subvolume) return parentItem->uuid;
+    return uuid;
+}
 QString DeviceItem::mappedDevice() const
 {
     const DeviceItem *twit = this;
@@ -1867,53 +1942,57 @@ QStringList DeviceItem::allowedUsesFor(bool real, bool all) const
 {
     if (!isVolume() && type != Subvolume) return QStringList();
     QStringList list;
-    if (!partman || size >= partman->rootSpaceNeeded) list << "root";
-    if (type == Subvolume) list << "home"; // swap requires Linux 5.0 or later
+    auto checkAndAdd = [&](const QString &use) {
+        const QString &realUse = realUseFor(use);
+        if (all || !partman || size >= partman->volSpecs.value(realUse).minimum) {
+            list.append(real ? realUse : use);
+        }
+    };
+
+    if (type != Subvolume) checkAndAdd("Format");
+    checkAndAdd("root");
+    if (type == Subvolume) checkAndAdd("home"); // swap requires Linux 5.0 or later
     else {
-        list.prepend("Format");
         if (type != VirtualBD) {
-            if (all || size <= 16*MB) list << "BIOS-GRUB";
+            if (all || size <= 16*MB) checkAndAdd("BIOS-GRUB");
             if (all || size <= 8*GB) {
-                if (size < (2*TB - 512)) list << "ESP";
-                list << "boot";
+                if (size < (2*TB - 512)) checkAndAdd("ESP");
+                checkAndAdd("boot");
             }
         }
-        list << "swap" << "home";
-    }
-    if (real) {
-        for(QString &use : list) use = realUseFor(use);
+        checkAndAdd("swap");
+        checkAndAdd("home");
     }
     return list;
 }
 QStringList DeviceItem::allowedFormats() const
 {
     QStringList list;
-    bool allowPreserve = false, selPreserve = false;
+    const QString &use = realUseFor();
+    bool allowPreserve = false;
     if (type == Subvolume) list.append("CREATE");
     else if (isVolume()) {
-        const QString &use = realUseFor();
         if (use.isEmpty()) return QStringList();
         else if (use == "/boot") list.append("ext4");
         else if (use == "BIOS-GRUB") list.append("GRUB");
         else if (use == "ESP") {
             list.append("FAT32");
-            if (size <= 4294901760) list.append("FAT16");
-            if (size <= 33553920) list.append("FAT12");
-            selPreserve = allowPreserve = (list.contains(curFormat, Qt::CaseInsensitive)
+            if (size <= (4*GB - 64*KB)) list.append("FAT16");
+            if (size <= (32*MB - 512)) list.append("FAT12");
+            allowPreserve = (list.contains(curFormat, Qt::CaseInsensitive)
                 || !curFormat.compare("VFAT", Qt::CaseInsensitive));
         } else if (use == "SWAP") {
             list.append("SWAP");
-            selPreserve = allowPreserve = list.contains(curFormat, Qt::CaseInsensitive);
+            allowPreserve = list.contains(curFormat, Qt::CaseInsensitive);
         } else {
             list << "ext4" << "ext3" << "ext2";
             list << "f2fs" << "jfs" << "xfs" << "btrfs";
             if (use != "FORMAT") allowPreserve = list.contains(curFormat, Qt::CaseInsensitive);
-            if (use == "/home") selPreserve = allowPreserve;
         }
     }
     if (encrypt) allowPreserve = false;
     if (allowPreserve) {
-        if (selPreserve) list.prepend("PRESERVE");
+        if (use != "/") list.prepend("PRESERVE");
         else list.append("PRESERVE");
     }
     return list;
@@ -1937,7 +2016,7 @@ long long DeviceItem::driveFreeSpace(bool inclusive) const
 {
     const DeviceItem *drvit = parent();
     if (!drvit) drvit = this;
-    long long free = drvit->size - (PARTMAN_SAFETY_MB * 1048576);
+    long long free = drvit->size - PARTMAN_SAFETY;
     for (int ixi = drvit->children.count() - 1; ixi >= 0; --ixi) {
         DeviceItem *partit = drvit->children.at(ixi);
         if (inclusive || partit != this) free -= partit->size;
@@ -1945,12 +2024,11 @@ long long DeviceItem::driveFreeSpace(bool inclusive) const
     return free;
 }
 /* Convenience */
-DeviceItem *DeviceItem::addPart(int defaultMB, const QString &defaultUse, bool crypto)
+DeviceItem *DeviceItem::addPart(long long defaultSize, const QString &defaultUse, bool crypto)
 {
     DeviceItem *partit = new DeviceItem(DeviceItem::Partition, this);
     if (!defaultUse.isEmpty()) partit->usefor = defaultUse;
-    partit->size = defaultMB;
-    partit->size *= 1048576;
+    partit->size = defaultSize;
     partit->autoFill();
     if (partit->canEncrypt()) partit->encrypt = crypto;
     if (partman) partman->notifyChange(partit);
@@ -1999,7 +2077,7 @@ void DeviceItem::autoFill(unsigned int changed)
             }
             label = newLabel;
         } else if (partman) {
-            label = partman->defaultLabels.value(use);
+            label = partman->volSpecs.value(use).defaultLabel;
         }
         // Automatic default boot device selection
         if ((type != VirtualBD) && (use == "/boot" || use == "/")) {
@@ -2047,59 +2125,6 @@ void DeviceItem::labelParts()
     if (partman) partman->resizeColumnsToFit();
 }
 
-int DeviceItem::layoutDefault(int rootPercent, bool crypto, bool updateTree)
-{
-    assert (partman != nullptr);
-    if (rootPercent<0) rootPercent = partman->gui.sliderPart->value();
-    if (updateTree) clear();
-    const long long driveSize = size / 1048576;
-    int rootFormatSize = static_cast<int>(driveSize - PARTMAN_SAFETY_MB);
-
-    // Boot partitions.
-    if (partman->proc.detectEFI()) {
-        if (updateTree) addPart(256, "ESP", crypto);
-        rootFormatSize -= 256;
-    } else if (driveSize >= 2097152 || partman->gptoverride) {
-        if (updateTree) addPart(1, "BIOS-GRUB", crypto);
-        rootFormatSize -= 1;
-    }
-    int rootMinMB = static_cast<int>(partman->rootSpaceNeeded / 1048576);
-    const int bootMinMB = static_cast<int>(partman->bootSpaceNeeded / 1048576);
-    if (!crypto) rootMinMB += bootMinMB;
-    else {
-        int bootFormatSize = 1024;
-        if (bootFormatSize < bootMinMB) bootFormatSize = static_cast<int>(partman->bootSpaceNeeded);
-        if (updateTree) addPart(bootFormatSize, "boot", crypto);
-        rootFormatSize -= bootFormatSize;
-    }
-    // Swap space.
-    int swapFormatSize = rootFormatSize-rootMinMB;
-    struct sysinfo sinfo;
-    if (sysinfo(&sinfo) != 0) sinfo.totalram = 2048;
-    else sinfo.totalram = (sinfo.totalram / (1048576 * 2)) * 3; // 1.5xRAM
-    sinfo.totalram /= 128; ++sinfo.totalram; sinfo.totalram *= 128; // Multiple of 128MB
-    if (swapFormatSize > static_cast<int>(sinfo.totalram)) swapFormatSize = static_cast<int>(sinfo.totalram);
-    int swapMaxMB = rootFormatSize / (20 * 128); ++swapMaxMB; swapMaxMB *= 128; // 5% root
-    if (swapMaxMB > 8192) swapMaxMB = 8192; // 8GB cap for the whole calculation.
-    if (swapFormatSize > swapMaxMB) swapFormatSize = swapMaxMB;
-    rootFormatSize -= swapFormatSize;
-    // Home
-    int homeFormatSize = rootFormatSize;
-    rootFormatSize = (rootFormatSize * rootPercent) / 100;
-    if (rootFormatSize < rootMinMB) rootFormatSize = rootMinMB;
-    homeFormatSize -= rootFormatSize;
-
-    if (updateTree) {
-        addPart(rootFormatSize, "root", crypto);
-        if (swapFormatSize>0) addPart(swapFormatSize, "swap", crypto);
-        if (homeFormatSize>0) addPart(homeFormatSize, "home", crypto);
-        labelParts();
-        driveAutoSetActive();
-        partman->treeSelChange();
-    }
-    return rootFormatSize;
-}
-
 // Return block device info that is suitable for a combo box.
 void DeviceItem::addToCombo(QComboBox *combo, bool warnNasty) const
 {
@@ -2274,8 +2299,8 @@ void DeviceItemDelegate::setEditorData(QWidget *editor, const QModelIndex &index
     case PartMan::Size:
         {
             QSpinBox *spin = qobject_cast<QSpinBox *>(editor);
-            spin->setRange(0, static_cast<int>(item->driveFreeSpace() / 1048576));
-            spin->setValue(item->size / 1048576);
+            spin->setRange(0, static_cast<int>(item->driveFreeSpace() / MB));
+            spin->setValue(item->size / MB);
         }
         break;
     case PartMan::UseFor:
